# Copyright (c) 2022-2025, Keivan Tafakkori. All rights reserved.
# See the file LICENSE file for licensing details.

def generate_model(solver_name, solver_options):

    module_mappings = {

        'orig-bbo': ('mealpy.bio_based', 'BBO', 'OriginalBBO'),
        'dev-bbo': ('mealpy.bio_based', 'BBO', 'DevBBO'),
        'orig-bboa': ('mealpy.bio_based', 'BBOA', 'OriginalBBOA'),
        'orig-bmo': ('mealpy.bio_based', 'BMO', 'OriginalBMO'),
        'orig-eoa': ('mealpy.bio_based', 'EOA', 'OriginalEOA'),
        'orig-iwo': ('mealpy.bio_based', 'IWO', 'OriginalIWO'),
        'dev-sbo': ('mealpy.bio_based', 'SBO', 'DevSBO'),
        'orig-sbo': ('mealpy.bio_based', 'SBO', 'OriginalSBO'),
        'dev-sma': ('mealpy.bio_based', 'SMA', 'DevSMA'),
        'orig-sma': ('mealpy.bio_based', 'SMA', 'OriginalSMA'),
        'dev-soa': ('mealpy.bio_based', 'SOA', 'DevSOA'),
        'orig-soa': ('mealpy.bio_based', 'SOA', 'OriginalSOA'),
        'orig-sos': ('mealpy.bio_based', 'SOS', 'OriginalSOS'),
        'dev-tpo': ('mealpy.bio_based', 'TPO', 'DevTPO'),
        'orig-tsa': ('mealpy.bio_based', 'TSA', 'OriginalTSA'),
        'dev-vcs': ('mealpy.bio_based', 'VCS', 'DevVCS'),
        'orig-vcs': ('mealpy.bio_based', 'VCS', 'OriginalVCS'),
        'orig-who': ('mealpy.bio_based', 'WHO', 'OriginalWHO'),
        'orig-cro': ('mealpy.evolutionary_based', 'CRO', 'OriginalCRO'),
        'ocro-cro': ('mealpy.evolutionary_based', 'CRO', 'OCRO'),
        'orig-de': ('mealpy.evolutionary_based', 'DE', 'OriginalDE'),
        'jade-de': ('mealpy.evolutionary_based', 'DE', 'JADE'),
        'sade-de': ('mealpy.evolutionary_based', 'DE', 'SADE'),
        'sap_de-de': ('mealpy.evolutionary_based', 'DE', 'SAP_DE'),
        'orig-ep': ('mealpy.evolutionary_based', 'EP', 'OriginalEP'),
        'levy-ep': ('mealpy.evolutionary_based', 'EP', 'LevyEP'),
        'orig-es': ('mealpy.evolutionary_based', 'ES', 'OriginalES'),
        'levy-es': ('mealpy.evolutionary_based', 'ES', 'LevyES'),
        'cma_es-es': ('mealpy.evolutionary_based', 'ES', 'CMA_ES'),
        'simple_cma_es-es': ('mealpy.evolutionary_based', 'ES', 'Simple_CMA_ES'),
        'orig-fpa': ('mealpy.evolutionary_based', 'FPA', 'OriginalFPA'),
        'base-ga': ('mealpy.evolutionary_based', 'GA', 'BaseGA'),
        'singlega-ga': ('mealpy.evolutionary_based', 'GA', 'SingleGA'),
        'elitesinglega-ga': ('mealpy.evolutionary_based', 'GA', 'EliteSingleGA'),
        'multiga-ga': ('mealpy.evolutionary_based', 'GA', 'MultiGA'),
        'elitemultiga-ga': ('mealpy.evolutionary_based', 'GA', 'EliteMultiGA'),
        'orig-ma': ('mealpy.evolutionary_based', 'MA', 'OriginalMA'),
        'orig-shade': ('mealpy.evolutionary_based', 'SHADE', 'OriginalSHADE'),
        'l_shade-shade': ('mealpy.evolutionary_based', 'SHADE', 'L_SHADE'),
        'dev-bro': ('mealpy.human_based', 'BRO', 'DevBRO'),
        'orig-bro': ('mealpy.human_based', 'BRO', 'OriginalBRO'),
        'dev-bso': ('mealpy.human_based', 'BSO', 'ImprovedBSO'),
        'orig-bso': ('mealpy.human_based', 'BSO', 'OriginalBSO'),
        'orig-ca': ('mealpy.human_based', 'CA', 'OriginalCA'),
        'orig-chio': ('mealpy.human_based', 'CHIO', 'OriginalCHIO'),
        'dev-chio': ('mealpy.human_based', 'CHIO', 'DevCHIO'),
        'dev-fbio': ('mealpy.human_based', 'FBIO', 'DevFBIO'),
        'orig-fbio': ('mealpy.human_based', 'FBIO', 'OriginalFBIO'),
        'dev-gska': ('mealpy.human_based', 'GSKA', 'DevGSKA'),
        'orig-gska': ('mealpy.human_based', 'GSKA', 'OriginalGSKA'),
        'orig-hbo': ('mealpy.human_based', 'HBO', 'OriginalHBO'),
        'orig-hco': ('mealpy.human_based', 'HCO', 'OriginalHCO'),
        'orig-ica': ('mealpy.human_based', 'ICA', 'OriginalICA'),
        'orig-lco': ('mealpy.human_based', 'LCO', 'OriginalLCO'),
        'dev-lco': ('mealpy.human_based', 'LCO', 'ImprovedLCO'),
        'dev-qsa': ('mealpy.human_based', 'QSA', 'ImprovedQSA'),
        'oppoqsa-qsa': ('mealpy.human_based', 'QSA', 'OppoQSA'),
        'levy-qsa': ('mealpy.human_based', 'QSA', 'LevyQSA'),
        'orig-qsa': ('mealpy.human_based', 'QSA', 'OriginalQSA'),
        'dev-saro': ('mealpy.human_based', 'SARO', 'DevSARO'),
        'orig-saro': ('mealpy.human_based', 'SARO', 'OriginalSARO'),
        'orig-spbo': ('mealpy.human_based', 'SPBO', 'OriginalSPBO'),
        'dev-spbo': ('mealpy.human_based', 'SPBO', 'DevSPBO'),
        'orig-ssdo': ('mealpy.human_based', 'SSDO', 'OriginalSSDO'),
        'dev-tlo': ('mealpy.human_based', 'TLO', 'ImprovedTLO'),
        'orig-tlo': ('mealpy.human_based', 'TLO', 'OriginalTLO'),
        'orig-toa': ('mealpy.human_based', 'TOA', 'OriginalTOA'),
        'orig-warso': ('mealpy.human_based', 'WarSO', 'OriginalWarSO'),
        'orig-aoa': ('mealpy.math_based', 'AOA', 'OriginalAOA'),
        'orig-cem': ('mealpy.math_based', 'CEM', 'OriginalCEM'),
        'orig-cgo': ('mealpy.math_based', 'CGO', 'OriginalCGO'),
        'orig-circlesa': ('mealpy.math_based', 'CircleSA', 'OriginalCircleSA'),
        'orig-gbo': ('mealpy.math_based', 'GBO', 'OriginalGBO'),
        'orig-hc': ('mealpy.math_based', 'HC', 'OriginalHC'),
        'swarmhc-hc': ('mealpy.math_based', 'HC', 'SwarmHC'),
        'orig-info': ('mealpy.math_based', 'INFO', 'OriginalINFO'),
        'orig-pss': ('mealpy.math_based', 'PSS', 'OriginalPSS'),
        'orig-run': ('mealpy.math_based', 'RUN', 'OriginalRUN'),
        'dev-sca': ('mealpy.math_based', 'SCA', 'DevSCA'),
        'orig-sca': ('mealpy.math_based', 'SCA', 'OriginalSCA'),
        'qtable-sca': ('mealpy.math_based', 'SCA', 'QTable'),
        'qlesca-sca': ('mealpy.math_based', 'SCA', 'QleSCA'),
        'orig-shio': ('mealpy.math_based', 'SHIO', 'OriginalSHIO'),
        'orig-ts': ('mealpy.math_based', 'TS', 'OriginalTS'),
        'dev-hs': ('mealpy.music_based', 'HS', 'DevHS'),
        'orig-hs': ('mealpy.music_based', 'HS', 'OriginalHS'),
        'orig-archoa': ('mealpy.physics_based', 'ArchOA', 'OriginalArchOA'),
        'orig-aso': ('mealpy.physics_based', 'ASO', 'OriginalASO'),
        'orig-cdo': ('mealpy.physics_based', 'CDO', 'OriginalCDO'),
        'dev-efo': ('mealpy.physics_based', 'EFO', 'DevEFO'),
        'orig-efo': ('mealpy.physics_based', 'EFO', 'OriginalEFO'),
        'orig-eo': ('mealpy.physics_based', 'EO', 'OriginalEO'),
        'mod-eo': ('mealpy.physics_based', 'EO', 'ModifiedEO'),
        'adaptiveeo-eo': ('mealpy.physics_based', 'EO', 'AdaptiveEO'),
        'orig-evo': ('mealpy.physics_based', 'EVO', 'OriginalEVO'),
        'orig-fla': ('mealpy.physics_based', 'FLA', 'OriginalFLA'),
        'orig-hgso': ('mealpy.physics_based', 'HGSO', 'OriginalHGSO'),
        'dev-mvo': ('mealpy.physics_based', 'MVO', 'DevMVO'),
        'orig-mvo': ('mealpy.physics_based', 'MVO', 'OriginalMVO'),
        'orig-nro': ('mealpy.physics_based', 'NRO', 'OriginalNRO'),
        'orig-rime': ('mealpy.physics_based', 'RIME', 'OriginalRIME'),
        'orig-sa': ('mealpy.physics_based', 'SA', 'OriginalSA'),
        'gaussiansa-sa': ('mealpy.physics_based', 'SA', 'GaussianSA'),
        'swarmsa-sa': ('mealpy.physics_based', 'SA', 'SwarmSA'),
        'orig-two': ('mealpy.physics_based', 'TWO', 'OriginalTWO'),
        'oppotwo-two': ('mealpy.physics_based', 'TWO', 'OppoTWO'),
        'levy-two': ('mealpy.physics_based', 'TWO', 'LevyTWO'),
        'enh-two': ('mealpy.physics_based', 'TWO', 'EnhancedTWO'),
        'orig-wdo': ('mealpy.physics_based', 'WDO', 'OriginalWDO'),
        'orig-abc': ('mealpy.swarm_based', 'ABC', 'OriginalABC'),
        'orig-acor': ('mealpy.swarm_based', 'ACOR', 'OriginalACOR'),
        'orig-agto': ('mealpy.swarm_based', 'AGTO', 'OriginalAGTO'),
        'mgto-agto': ('mealpy.swarm_based', 'AGTO', 'MGTO'),
        'orig-alo': ('mealpy.swarm_based', 'ALO', 'OriginalALO'),
        'dev-alo': ('mealpy.swarm_based', 'ALO', 'DevALO'),
        'orig-ao': ('mealpy.swarm_based', 'AO', 'OriginalAO'),
        'orig-aro': ('mealpy.swarm_based', 'ARO', 'OriginalARO'),
        'laro-aro': ('mealpy.swarm_based', 'ARO', 'LARO'),
        'iaro-aro': ('mealpy.swarm_based', 'ARO', 'IARO'),
        'orig-avoa': ('mealpy.swarm_based', 'AVOA', 'OriginalAVOA'),
        'orig-ba': ('mealpy.swarm_based', 'BA', 'OriginalBA'),
        'adaptiveba-ba': ('mealpy.swarm_based', 'BA', 'AdaptiveBA'),
        'dev-ba': ('mealpy.swarm_based', 'BA', 'DevBA'),
        'cleverbookbeesa-beesa': ('mealpy.swarm_based', 'BeesA', 'CleverBookBeesA'),
        'orig-beesa': ('mealpy.swarm_based', 'BeesA', 'OriginalBeesA'),
        'probbeesa-beesa': ('mealpy.swarm_based', 'BeesA', 'ProbBeesA'),
        'orig-bes': ('mealpy.swarm_based', 'BES', 'OriginalBES'),
        'orig-bfo': ('mealpy.swarm_based', 'BFO', 'OriginalBFO'),
        'abfo-bfo': ('mealpy.swarm_based', 'BFO', 'ABFO'),
        'orig-bsa': ('mealpy.swarm_based', 'BSA', 'OriginalBSA'),
        'orig-coa': ('mealpy.swarm_based', 'COA', 'OriginalCOA'),
        'orig-coatioa': ('mealpy.swarm_based', 'CoatiOA', 'OriginalCoatiOA'),
        'orig-csa': ('mealpy.swarm_based', 'CSA', 'OriginalCSA'),
        'orig-cso': ('mealpy.swarm_based', 'CSO', 'OriginalCSO'),
        'orig-dmoa': ('mealpy.swarm_based', 'DMOA', 'OriginalDMOA'),
        'dev-dmoa': ('mealpy.swarm_based', 'DMOA', 'DevDMOA'),
        'orig-do': ('mealpy.swarm_based', 'DO', 'OriginalDO'),
        'orig-eho': ('mealpy.swarm_based', 'EHO', 'OriginalEHO'),
        'orig-esoa': ('mealpy.swarm_based', 'ESOA', 'OriginalESOA'),
        'orig-fa': ('mealpy.swarm_based', 'FA', 'OriginalFA'),
        'orig-ffa': ('mealpy.swarm_based', 'FFA', 'OriginalFFA'),
        'orig-ffo': ('mealpy.swarm_based', 'FFO', 'OriginalFFO'),
        'orig-foa': ('mealpy.swarm_based', 'FOA', 'OriginalFOA'),
        'dev-foa': ('mealpy.swarm_based', 'FOA', 'DevFOA'),
        'whalefoa-foa': ('mealpy.swarm_based', 'FOA', 'WhaleFOA'),
        'orig-fox': ('mealpy.swarm_based', 'FOX', 'OriginalFOX'),
        'dev-fox': ('mealpy.swarm_based', 'FOX', 'DevFOX'),
        'orig-gjo': ('mealpy.swarm_based', 'GJO', 'OriginalGJO'),
        'orig-goa': ('mealpy.swarm_based', 'GOA', 'OriginalGOA'),
        'orig-gto': ('mealpy.swarm_based', 'GTO', 'OriginalGTO'),
        'matlab102-gto': ('mealpy.swarm_based', 'GTO', 'Matlab102GTO'),
        'matlab101-gto': ('mealpy.swarm_based', 'GTO', 'Matlab101GTO'),
        'orig-gwo': ('mealpy.swarm_based', 'GWO', 'OriginalGWO'),
        'rw_gwo-gwo': ('mealpy.swarm_based', 'GWO', 'RW_GWO'),
        'gwo_woa-gwo': ('mealpy.swarm_based', 'GWO', 'GWO_WOA'),
        'igwo-gwo': ('mealpy.swarm_based', 'GWO', 'IGWO'),
        'orig-hba': ('mealpy.swarm_based', 'HBA', 'OriginalHBA'),
        'orig-hgs': ('mealpy.swarm_based', 'HGS', 'OriginalHGS'),
        'orig-hho': ('mealpy.swarm_based', 'HHO', 'OriginalHHO'),
        'dev-ja': ('mealpy.swarm_based', 'JA', 'DevJA'),
        'orig-ja': ('mealpy.swarm_based', 'JA', 'OriginalJA'),
        'levy-ja': ('mealpy.swarm_based', 'JA', 'LevyJA'),
        'orig-mfo': ('mealpy.swarm_based', 'MFO', 'OriginalMFO'),
        'orig-mgo': ('mealpy.swarm_based', 'MGO', 'OriginalMGO'),
        'orig-mpa': ('mealpy.swarm_based', 'MPA', 'OriginalMPA'),
        'orig-mrfo': ('mealpy.swarm_based', 'MRFO', 'OriginalMRFO'),
        'wmqimrfo-mrfo': ('mealpy.swarm_based', 'MRFO', 'WMQIMRFO'),
        'orig-msa': ('mealpy.swarm_based', 'MSA', 'OriginalMSA'),
        'orig-ngo': ('mealpy.swarm_based', 'NGO', 'OriginalNGO'),
        'orig-nmra': ('mealpy.swarm_based', 'NMRA', 'OriginalNMRA'),
        'dev-nmra': ('mealpy.swarm_based', 'NMRA', 'ImprovedNMRA'),
        'orig-ooa': ('mealpy.swarm_based', 'OOA', 'OriginalOOA'),
        'orig-pfa': ('mealpy.swarm_based', 'PFA', 'OriginalPFA'),
        'orig-poa': ('mealpy.swarm_based', 'POA', 'OriginalPOA'),
        'orig-pso': ('mealpy.swarm_based', 'PSO', 'OriginalPSO'),
        'aiw_pso-pso': ('mealpy.swarm_based', 'PSO', 'AIW_PSO'),
        'ldw_pso-pso': ('mealpy.swarm_based', 'PSO', 'LDW_PSO'),
        'p_pso-pso': ('mealpy.swarm_based', 'PSO', 'P_PSO'),
        'hpso_tvac-pso': ('mealpy.swarm_based', 'PSO', 'HPSO_TVAC'),
        'c_pso-pso': ('mealpy.swarm_based', 'PSO', 'C_PSO'),
        'cl_pso-pso': ('mealpy.swarm_based', 'PSO', 'CL_PSO'),
        'orig-scso': ('mealpy.swarm_based', 'SCSO', 'OriginalSCSO'),
        'orig-seaho': ('mealpy.swarm_based', 'SeaHO', 'OriginalSeaHO'),
        'orig-servaloa': ('mealpy.swarm_based', 'ServalOA', 'OriginalServalOA'),
        'orig-sfo': ('mealpy.swarm_based', 'SFO', 'OriginalSFO'),
        'dev-sfo': ('mealpy.swarm_based', 'SFO', 'ImprovedSFO'),
        'orig-sho': ('mealpy.swarm_based', 'SHO', 'OriginalSHO'),
        'orig-slo': ('mealpy.swarm_based', 'SLO', 'OriginalSLO'),
        'mod-slo': ('mealpy.swarm_based', 'SLO', 'ModifiedSLO'),
        'dev-slo': ('mealpy.swarm_based', 'SLO', 'ImprovedSLO'),
        'orig-srsr': ('mealpy.swarm_based', 'SRSR', 'OriginalSRSR'),
        'dev-ssa': ('mealpy.swarm_based', 'SSA', 'DevSSA'),
        'orig-ssa': ('mealpy.swarm_based', 'SSA', 'OriginalSSA'),
        'orig-sso': ('mealpy.swarm_based', 'SSO', 'OriginalSSO'),
        'orig-sspidera': ('mealpy.swarm_based', 'SSpiderA', 'OriginalSSpiderA'),
        'orig-sspidero': ('mealpy.swarm_based', 'SSpiderO', 'OriginalSSpiderO'),
        'orig-sto': ('mealpy.swarm_based', 'STO', 'OriginalSTO'),
        'orig-tdo': ('mealpy.swarm_based', 'TDO', 'OriginalTDO'),
        'orig-tso': ('mealpy.swarm_based', 'TSO', 'OriginalTSO'),
        'orig-waoa': ('mealpy.swarm_based', 'WaOA', 'OriginalWaOA'),
        'orig-woa': ('mealpy.swarm_based', 'WOA', 'OriginalWOA'),
        'hi_woa-woa': ('mealpy.swarm_based', 'WOA', 'HI_WOA'),
        'orig-zoa': ('mealpy.swarm_based', 'ZOA', 'OriginalZOA'),
        'orig-aeo': ('mealpy.system_based', 'AEO', 'OriginalAEO'),
        'dev-aeo': ('mealpy.system_based', 'AEO', 'ImprovedAEO'),
        'enh-aeo': ('mealpy.system_based', 'AEO', 'EnhancedAEO'),
        'mod-aeo': ('mealpy.system_based', 'AEO', 'ModifiedAEO'),
        'augmented-aeo': ('mealpy.system_based', 'AEO', 'AugmentedAEO'),
        'dev-gco': ('mealpy.system_based', 'GCO', 'DevGCO'),
        'orig-gco': ('mealpy.system_based', 'GCO', 'OriginalGCO'),
        'orig-wca': ('mealpy.system_based', 'WCA', 'OriginalWCA'),
    }

    module_name, class_name, model_name = module_mappings.get(solver_name, (None, None, None))

    if module_name and class_name and model_name:
    
        module = __import__(module_name, fromlist=[class_name])
        model_class = getattr(module, class_name)
        model_object = getattr(model_class, model_name)(**solver_options)

    else:
    
        raise ValueError("Invalid solver name. Please refer to https://feloopy.readthedocs.io/en/latest/heuristic.html")

    return model_object